
% Tasso di rendimento interno di una obbligazione con prezzo P e cedole c (%, semestrali, il tasso c e' annuo) , maturit T (in anni, T>=1)
% e valore nominale R. con capitalizzazione composta continuamente

% unita' di misura temporale 1 = 1 anno

% algoritmo di Newton-Rapson per f(y)=0

%  y_n+1=y_n - f(y_n)/f'(y_n) con y(0)=valore di input iniziale.

% f(y)=C (exp(-yt_1)+exp(-yt_2)+ ....... + exp(-y t_m))+ R*exp(-y t_m) - P

% derivatata prima

% f'(y)=-C(t_1 *exp(-yt_1)+ ..... + t_m *exp(-yt_m))-R*t_m *exp(-y t_m)

% l'arresto dell'algoritmo avviene quando |f(y_n)|<tol.

% Poiche' f'(y)<0 e f(+infinito)=-P<0

% perche' vi sia una soluzione in [0,+infinito] all'equazione f(y)=0, deve succedere che f(0)>0, ma:

% C*m+R>=P (questa e' introdotta come una condizione di controllo nel programma).

% derivatata prima

% f'(y)=-C(t_1 *exp(-yt_1)+ ..... + t_m *exp(-yt_m))-R*t_m *exp(-y t_m)

% sulla base della scelta dell'unita' di misura temporale avremo

% t_0=0 (data di osservazione del prezzo) t_1=0.5 t_2=1, t_3=1.5, .... t_k=0.5*k, t_m=T=m*0.5   

%---------------------------------------------------------------------------------------------------------------
% In output il programma restituisce il tasso di rendimento approssimato y (in base a tol) e il valore in y
% della funzione f che esprime lo scarto tra il prezzo teorico dell'obbligazione e il prezzo osservato.
%---------------------------------------------------------------------------------------------------------------

function [y,z]=TIR(P,R,c,T,tol)

y=[];

z=[];

% importo della cedola

C=(R*c)/200; % il tasso e' annuo e la cedola e' pagata semestralmente.

% numero di scadenze

m=2.*T;

% condizione di controllo sui dati immessi

if (C*m+R>=P) % la condizione di controllo e' soddisfatta e procedo con l'algoritmo.
    
% Algoritmo di Newton-Rapson:  

% valore iniziale

y=0;

% valore della funzione scarto f in y=0.

z=C*m+R-P;

 while (abs(z)>=tol)

        
    [z,Dz]=f(y,m,C,R,P);
     
    y=y-z/Dz; 
    
    clear Dz;
    
 end   
    
else
    
    
    disp('i dati immessi non sono corretti C*m+R');

   break;
   
end   

%---------------------------------------------------------------------------------------------------------------
% funzione f a cui applicare l'alg. di Newton-Rapson, e la sua derivata.
%---------------------------------------------------------------------------------------------------------------

function [z,Dz]=f(y,m,C,R,P)


z=0;

Dz=0;


for i=1:1:m
    
    t(i)=i*0.5;


    z=z+(C*exp(-y*t(i)));

    
    Dz=Dz-(C*t(i)*exp(-y*t(i)));
    
end

    
z=z+R*exp(-y* t(m))-P;
    
Dz=Dz-R*t(m)*exp(-y*t(m));

